==> ui-18.1.4.tgz: package/components/transfer/Transfer.js <==
var __rest = (this && this.__rest) || function (s, e) {
    var t = {};
    for (var p in s) if (Object.prototype.hasOwnProperty.call(s, p) && e.indexOf(p) < 0)
        t[p] = s[p];
    if (s != null && typeof Object.getOwnPropertySymbols === "function")
        for (var i = 0, p = Object.getOwnPropertySymbols(s); i < p.length; i++) {
            if (e.indexOf(p[i]) < 0 && Object.prototype.propertyIsEnumerable.call(s, p[i]))
                t[p[i]] = s[p[i]];
        }
    return t;
};
import { jsx as _jsx, jsxs as _jsxs } from "react/jsx-runtime";
import React, { useCallback, useMemo } from 'react';
import { LeftOutlined, RightOutlined } from '@react-devui/icons';
import { getClassName } from '@react-devui/utils';
import { useGeneralContext, useDValue } from '../../hooks';
import { registerComponentMate } from '../../utils';
import { DBaseDesign } from '../_base-design';
import { DButton } from '../button';
import { useFormControl } from '../form';
import { useComponentConfig, usePrefixConfig } from '../root';
import { DPanel } from './Panel';
export const IS_SELECTED = Symbol();
const { COMPONENT_NAME } = registerComponentMate({ COMPONENT_NAME: 'DTransfer' });
export function DTransfer(props) {
    const _a = useComponentConfig(COMPONENT_NAME, props), { dFormControl, dList, dModel, dSelected, dTitle, dActions = ['right', 'left'], dLoading = [false, false], dSearchable = false, dSearchValue, dDisabled = false, dCustomItem, dCustomSearch, onModelChange, onSelectedChange, onSearchValueChange, onScrollBottom } = _a, restProps = __rest(_a, ["dFormControl", "dList", "dModel", "dSelected", "dTitle", "dActions", "dLoading", "dSearchable", "dSearchValue", "dDisabled", "dCustomItem", "dCustomSearch", "onModelChange", "onSelectedChange", "onSearchValueChange", "onScrollBottom"]);
    //#region Context
    const dPrefix = usePrefixConfig();
    const { gDisabled } = useGeneralContext();
    //#endregion
    const itemsMap = useMemo(() => new Map(dList.map((item) => [item.value, item])), [dList]);
    const formControlInject = useFormControl(dFormControl);
    const [_valueRight, changeValueRight] = useDValue([], dModel, (value) => {
        if (onModelChange) {
            onModelChange(value, value.map((v) => itemsMap.get(v)));
        }
    }, undefined, formControlInject);
    const valueRight = useMemo(() => new Set(_valueRight), [_valueRight]);
    const [_selected, changeSelected] = useDValue([], dSelected, (value) => {
        if (onSelectedChange) {
            onSelectedChange(value, value.map((v) => itemsMap.get(v)));
        }
    });
    const selected = useMemo(() => new Set(_selected), [_selected]);
    const disabled = dDisabled || gDisabled || (dFormControl === null || dFormControl === void 0 ? void 0 : dFormControl.control.disabled);
    const [searchValues, changeSearchValue] = useDValue(['', ''], dSearchValue, onSearchValueChange);
    const _filterFn = dCustomSearch === null || dCustomSearch === void 0 ? void 0 : dCustomSearch.filter;
    const filterFn = useCallback((searchStr, item) => {
        const defaultFilterFn = (item) => {
            return item.label.includes(searchStr);
        };
        return _filterFn ? _filterFn(searchStr, item) : defaultFilterFn(item);
    }, [_filterFn]);
    const sortFn = dCustomSearch === null || dCustomSearch === void 0 ? void 0 : dCustomSearch.sort;
    const [listLeft, listRight, selectedNumLeft, selectedNumRight, stateLeft, stateRight] = useMemo(() => {
        const listL = [];
        const listR = [];
        let selectedNumL = 0;
        let selectedNumR = 0;
        let emptyL = true;
        let hasSelectedL = false;
        let checkAllL = true;
        let emptyR = true;
        let hasSelectedR = false;
        let checkAllR = true;
        dList.forEach((item) => {
            const isLeft = !valueRight.has(item.value);
            const newItem = Object.assign({}, item);
            const searchValue = searchValues[isLeft ? 0 : 1];
            if (!searchValue || filterFn(searchValue, item)) {
                newItem[IS_SELECTED] = false;
                if (selected.has(item.value)) {
                    newItem[IS_SELECTED] = true;
                    isLeft ? (selectedNumL += 1) : (selectedNumR += 1);
                    if (!item.disabled) {
                        isLeft ? (hasSelectedL = true) : (hasSelectedR = true);
                    }
                }
                else {
                    if (!item.disabled) {
                        isLeft ? (checkAllL = false) : (checkAllR = false);
                    }
                }
                if (isLeft) {
                    listL.push(newItem);
                    if (!item.disabled) {
                        emptyL = false;
                    }
                }
                else {
                    listR.push(newItem);
                    if (!item.disabled) {
                        emptyR = false;
                    }
                }
            }
        });
        if (searchValues[0] && sortFn) {
            listL.sort(sortFn);
        }
        if (searchValues[1] && sortFn) {
            listR.sort(sortFn);
        }
        const stateL = emptyL ? false : checkAllL ? true : hasSelectedL ? 'mixed' : false;
        const stateR = emptyR ? false : checkAllR ? true : hasSelectedR ? 'mixed' : false;
        return [listL, listR, selectedNumL, selectedNumR, stateL, stateR];
    }, [dList, filterFn, searchValues, selected, sortFn, valueRight]);
    const handleSelectedChange = (val) => {
        changeSelected((draft) => {
            const index = draft.findIndex((v) => v === val);
            if (index !== -1) {
                draft.splice(index, 1);
            }
            else {
                draft.push(val);
            }
        });
    };
    const handleAllSelected = (isSelected, isLeft) => {
        changeSelected((draft) => {
            const newSelected = new Set(draft);
            for (const item of isLeft ? listLeft : listRight) {
                if (item.disabled) {
                    continue;
                }
                if (isSelected && !item[IS_SELECTED]) {
                    newSelected.add(item.value);
                }
                else if (!isSelected && item[IS_SELECTED]) {
                    newSelected.delete(item.value);
                }
            }
            return Array.from(newSelected);
        });
    };
    const handleButtonClick = (isLeft) => {
        changeValueRight((draft) => {
            const newValueRight = new Set(draft);
            (isLeft ? listLeft : listRight).forEach((item) => {
                if (item[IS_SELECTED]) {
                    if (isLeft) {
                        newValueRight.add(item.value);
                    }
                    else {
                        newValueRight.delete(item.value);
                    }
                }
            });
            return Array.from(newValueRight);
        });
        changeSelected((draft) => {
            const newSelected = new Set(draft);
            (isLeft ? listLeft : listRight).forEach((item) => {
                if (item[IS_SELECTED]) {
                    newSelected.delete(item.value);
                }
            });
            return Array.from(newSelected);
        });
    };
    return (_jsx(DBaseDesign, Object.assign({ dComposeDesign: false, dFormDesign: {
            control: dFormControl,
        } }, { children: ({ render: renderBaseDesign }) => {
            var _a, _b;
            return renderBaseDesign(_jsxs("div", Object.assign({}, restProps, { className: getClassName(restProps.className, `${dPrefix}transfer`, {
                    'is-disabled': disabled,
                }) }, { children: [_jsx(DPanel, { dList: listLeft, dSelectedNum: selectedNumLeft, dState: stateLeft, dTitle: dTitle === null || dTitle === void 0 ? void 0 : dTitle[0], dLoading: (_a = dLoading[0]) !== null && _a !== void 0 ? _a : false, dSearchable: dSearchable, dCustomItem: dCustomItem, onSelectedChange: handleSelectedChange, onAllSelected: (selected) => {
                            handleAllSelected(selected, true);
                        }, onSearch: (val) => {
                            changeSearchValue((draft) => {
                                draft[0] = val;
                            });
                        }, onScrollBottom: () => {
                            onScrollBottom === null || onScrollBottom === void 0 ? void 0 : onScrollBottom('left');
                        } }), _jsx("div", Object.assign({ className: `${dPrefix}transfer__actions` }, { children: React.Children.map(dActions, (action) => action === 'right' ? (_jsx(DButton, { disabled: stateLeft === false, onClick: () => {
                                handleButtonClick(true);
                            }, dType: "secondary", dIcon: _jsx(RightOutlined, {}) }, "$$right")) : action === 'left' ? (_jsx(DButton, { disabled: stateRight === false, onClick: () => {
                                handleButtonClick(false);
                            }, dType: "secondary", dIcon: _jsx(LeftOutlined, {}) }, "$$left")) : (action)) })), _jsx(DPanel, { dList: listRight, dSelectedNum: selectedNumRight, dState: stateRight, dTitle: dTitle === null || dTitle === void 0 ? void 0 : dTitle[1], dLoading: (_b = dLoading[1]) !== null && _b !== void 0 ? _b : false, dSearchable: dSearchable, dCustomItem: dCustomItem, onSelectedChange: handleSelectedChange, onAllSelected: (selected) => {
                            handleAllSelected(selected, false);
                        }, onSearch: (val) => {
                            changeSearchValue((draft) => {
                                draft[1] = val;
                            });
                        }, onScrollBottom: () => {
                            onScrollBottom === null || onScrollBottom === void 0 ? void 0 : onScrollBottom('right');
                        } })] })));
        } })));
}

==> ui-18.3.2.tgz: package/components/transfer/Transfer.js <==
var __rest = (this && this.__rest) || function (s, e) {
    var t = {};
    for (var p in s) if (Object.prototype.hasOwnProperty.call(s, p) && e.indexOf(p) < 0)
        t[p] = s[p];
    if (s != null && typeof Object.getOwnPropertySymbols === "function")
        for (var i = 0, p = Object.getOwnPropertySymbols(s); i < p.length; i++) {
            if (e.indexOf(p[i]) < 0 && Object.prototype.propertyIsEnumerable.call(s, p[i]))
                t[p[i]] = s[p[i]];
        }
    return t;
};
import { jsx as _jsx, jsxs as _jsxs } from "react/jsx-runtime";
import React, { useCallback, useMemo } from 'react';
import { LeftOutlined, RightOutlined } from '@react-devui/icons';
import { getClassName } from '@react-devui/utils';
import { useGeneralContext, useDValue } from '../../hooks';
import { registerComponentMate } from '../../utils';
import { DBaseDesign } from '../_base-design';
import { DButton } from '../button';
import { useFormControl } from '../form';
import { useComponentConfig, usePrefixConfig } from '../root';
import { DPanel } from './Panel';
export const IS_SELECTED = Symbol();
const { COMPONENT_NAME } = registerComponentMate({ COMPONENT_NAME: 'DTransfer' });
export function DTransfer(props) {
    const _a = useComponentConfig(COMPONENT_NAME, props), { dFormControl, dList, dModel, dSelected, dTitle, dActions = ['right', 'left'], dLoading = [false, false], dSearchable = false, dSearchValue, dDisabled = false, dVirtual = false, dCustomItem, dCustomSearch, onModelChange, onSelectedChange, onSearchValueChange, onScrollBottom } = _a, restProps = __rest(_a, ["dFormControl", "dList", "dModel", "dSelected", "dTitle", "dActions", "dLoading", "dSearchable", "dSearchValue", "dDisabled", "dVirtual", "dCustomItem", "dCustomSearch", "onModelChange", "onSelectedChange", "onSearchValueChange", "onScrollBottom"]);
    //#region Context
    const dPrefix = usePrefixConfig();
    const { gDisabled } = useGeneralContext();
    //#endregion
    const itemsMap = useMemo(() => new Map(dList.map((item) => [item.value, item])), [dList]);
    const formControlInject = useFormControl(dFormControl);
    const [_valueRight, changeValueRight] = useDValue([], dModel, (value) => {
        if (onModelChange) {
            onModelChange(value, value.map((v) => itemsMap.get(v)));
        }
    }, undefined, formControlInject);
    const valueRight = useMemo(() => new Set(_valueRight), [_valueRight]);
    const [_selected, changeSelected] = useDValue([], dSelected, (value) => {
        if (onSelectedChange) {
            onSelectedChange(value, value.map((v) => itemsMap.get(v)));
        }
    });
    const selected = useMemo(() => new Set(_selected), [_selected]);
    const disabled = dDisabled || gDisabled || (dFormControl === null || dFormControl === void 0 ? void 0 : dFormControl.control.disabled);
    const [searchValues, changeSearchValue] = useDValue(['', ''], dSearchValue, onSearchValueChange);
    const _filterFn = dCustomSearch === null || dCustomSearch === void 0 ? void 0 : dCustomSearch.filter;
    const filterFn = useCallback((searchStr, item) => {
        const defaultFilterFn = (item) => {
            return item.label.includes(searchStr);
        };
        return _filterFn ? _filterFn(searchStr, item) : defaultFilterFn(item);
    }, [_filterFn]);
    const sortFn = dCustomSearch === null || dCustomSearch === void 0 ? void 0 : dCustomSearch.sort;
    const [listLeft, listRight, selectedNumLeft, selectedNumRight, stateLeft, stateRight] = useMemo(() => {
        const listL = [];
        const listR = [];
        let selectedNumL = 0;
        let selectedNumR = 0;
        let emptyL = true;
        let hasSelectedL = false;
        let checkAllL = true;
        let emptyR = true;
        let hasSelectedR = false;
        let checkAllR = true;
        dList.forEach((item) => {
            const isLeft = !valueRight.has(item.value);
            const newItem = Object.assign({}, item);
            const searchValue = searchValues[isLeft ? 0 : 1];
            if (!searchValue || filterFn(searchValue, item)) {
                newItem[IS_SELECTED] = false;
                if (selected.has(item.value)) {
                    newItem[IS_SELECTED] = true;
                    isLeft ? (selectedNumL += 1) : (selectedNumR += 1);
                    if (!item.disabled) {
                        isLeft ? (hasSelectedL = true) : (hasSelectedR = true);
                    }
                }
                else {
                    if (!item.disabled) {
                        isLeft ? (checkAllL = false) : (checkAllR = false);
                    }
                }
                if (isLeft) {
                    listL.push(newItem);
                    if (!item.disabled) {
                        emptyL = false;
                    }
                }
                else {
                    listR.push(newItem);
                    if (!item.disabled) {
                        emptyR = false;
                    }
                }
            }
        });
        if (searchValues[0] && sortFn) {
            listL.sort(sortFn);
        }
        if (searchValues[1] && sortFn) {
            listR.sort(sortFn);
        }
        const stateL = emptyL ? false : checkAllL ? true : hasSelectedL ? 'mixed' : false;
        const stateR = emptyR ? false : checkAllR ? true : hasSelectedR ? 'mixed' : false;
        return [listL, listR, selectedNumL, selectedNumR, stateL, stateR];
    }, [dList, filterFn, searchValues, selected, sortFn, valueRight]);
    const handleSelectedChange = (val) => {
        changeSelected((draft) => {
            const index = draft.findIndex((v) => v === val);
            if (index !== -1) {
                draft.splice(index, 1);
            }
            else {
                draft.push(val);
            }
        });
    };
    const handleAllSelected = (isSelected, isLeft) => {
        changeSelected((draft) => {
            const newSelected = new Set(draft);
            for (const item of isLeft ? listLeft : listRight) {
                if (item.disabled) {
                    continue;
                }
                if (isSelected && !item[IS_SELECTED]) {
                    newSelected.add(item.value);
                }
                else if (!isSelected && item[IS_SELECTED]) {
                    newSelected.delete(item.value);
                }
            }
            return Array.from(newSelected);
        });
    };
    const handleButtonClick = (isLeft) => {
        changeValueRight((draft) => {
            const newValueRight = new Set(draft);
            (isLeft ? listLeft : listRight).forEach((item) => {
                if (item[IS_SELECTED]) {
                    if (isLeft) {
                        newValueRight.add(item.value);
                    }
                    else {
                        newValueRight.delete(item.value);
                    }
                }
            });
            return Array.from(newValueRight);
        });
        changeSelected((draft) => {
            const newSelected = new Set(draft);
            (isLeft ? listLeft : listRight).forEach((item) => {
                if (item[IS_SELECTED]) {
                    newSelected.delete(item.value);
                }
            });
            return Array.from(newSelected);
        });
    };
    return (_jsx(DBaseDesign, Object.assign({ dComposeDesign: false, dFormDesign: {
            control: dFormControl,
        } }, { children: ({ render: renderBaseDesign }) => {
            var _a, _b;
            return renderBaseDesign(_jsxs("div", Object.assign({}, restProps, { className: getClassName(restProps.className, `${dPrefix}transfer`, {
                    'is-disabled': disabled,
                }) }, { children: [_jsx(DPanel, { dList: listLeft, dSelectedNum: selectedNumLeft, dState: stateLeft, dTitle: dTitle === null || dTitle === void 0 ? void 0 : dTitle[0], dLoading: (_a = dLoading[0]) !== null && _a !== void 0 ? _a : false, dSearchable: dSearchable, dVirtual: dVirtual, dCustomItem: dCustomItem, onSelectedChange: handleSelectedChange, onAllSelected: (selected) => {
                            handleAllSelected(selected, true);
                        }, onSearch: (val) => {
                            changeSearchValue((draft) => {
                                draft[0] = val;
                            });
                        }, onScrollBottom: () => {
                            onScrollBottom === null || onScrollBottom === void 0 ? void 0 : onScrollBottom('left');
                        } }), _jsx("div", Object.assign({ className: `${dPrefix}transfer__actions` }, { children: React.Children.map(dActions, (action) => action === 'right' ? (_jsx(DButton, { disabled: stateLeft === false, onClick: () => {
                                handleButtonClick(true);
                            }, dType: "secondary", dIcon: _jsx(RightOutlined, {}) }, "$$right")) : action === 'left' ? (_jsx(DButton, { disabled: stateRight === false, onClick: () => {
                                handleButtonClick(false);
                            }, dType: "secondary", dIcon: _jsx(LeftOutlined, {}) }, "$$left")) : (action)) })), _jsx(DPanel, { dList: listRight, dSelectedNum: selectedNumRight, dState: stateRight, dTitle: dTitle === null || dTitle === void 0 ? void 0 : dTitle[1], dLoading: (_b = dLoading[1]) !== null && _b !== void 0 ? _b : false, dSearchable: dSearchable, dVirtual: dVirtual, dCustomItem: dCustomItem, onSelectedChange: handleSelectedChange, onAllSelected: (selected) => {
                            handleAllSelected(selected, false);
                        }, onSearch: (val) => {
                            changeSearchValue((draft) => {
                                draft[1] = val;
                            });
                        }, onScrollBottom: () => {
                            onScrollBottom === null || onScrollBottom === void 0 ? void 0 : onScrollBottom('right');
                        } })] })));
        } })));
}
